;******************************************************************************************************************
;    RX_GEN_HT6P_0_5V.ASM_EEPROM  (V5)
;
;   
;    RECEPTOR PARA CONTROLE REMOTO HT6P20B COM UMA SAIDA PWM DE 0-5V E OUTRA RETENO OU PULSO CONFORME JUMPER
;    NO PINO 6.
;
;    AUTOR: CLUDIO LRIOS
;
;    INCIO: 15-10-13     TRMINO : 15-10-13
;    ATUALIZADO EM:11-06-2014 (SALVAR NA EEPROM)
;    ATUALIZADO EM:14-09-2015 (BUG ELIMINADO)
;    ATUALIZADO EM:08-11-2015 (ALTERADO ROTINA DE SALVAMENTO NA EEPROM, TRAVA NO BOTO ON/OFF)
;    ATUALIZADO EM:11-12-2016 (ACRESCENTADO JUMPER DE OPO DE PULSO/RETENO NO PINO 6 AO GND
;                             SEM JUMPER= MODO PULSO NA SAIDA S3 - NO PINO 5
;                             COM JUMPER AO GND = MODO RETENO NA SAIDA S3 - NO PINO 5)
;
;    OBJETIVO: CONTROLAR A DISTANCIA, UMA SAIDA DO PIC EM MODO PWM EM 0 - 5 VOLTS
;    EM APLICAES DE DIMMER DC OU CONTROLE DE VOLUME, USANDO O BOTO DA ESQUERDA
;    PARA REDUZIR E O DA DIREITA PARA AUMENTAR A TENSO DE SAIDA DO PINO 7.
;    APERTANDO O 3 BOTO (OU OS 2 BOTES AO MESMO TEMPO) TEMOS A FUNO ON/OFF (RETENO)
;    NO PINO 5. 
;    USA MICROCONTROLADOR PIC12F675 DE 8 PINOS 

;    A FREQUNCIA DA SADA PWM  DE APROXIMADAMENTE  120 HZ.
;    UTILIZA CONTROLES REMOTOS CHAVEIRINHOS COM CI HT6P20B.
;    GRAVA NO ULTIMO ENDEREO DA EEPROM A ULTIMA ATUAO INC/DECREMENTO DO PWM
;    RECUPERANDO A INFORMAO AO RELIGAR A ALIMENTAO
;===================================================================================================================
;    GRAVAR CONTROLES:
;    APERTE O 'BOTO LEARN' E DEVER ACENDER O 'LED LEARN'.
;    APERTE UM BOTO DO CONTROLE E AO APRENDER APAGAR O 'LED LEARN'.
;    REPITA COM OS OUTROS BOTES DO CONTROLE (TEM QUE REPETIR COM TODOS OS BOTES DO 
;    CONTROLE). 
;    REPITA COM TODOS OS OUTROS CONTROLES.
;===================================================================================================================
;    APAGAR TODOS CONTROLES:
;    APERTE O 'BOTO LEARN' E MANTENHA APERTADO POR MAIS DE 5 SEGUNDOS.
;    QUANDO COMEAR A PISCAR O 'LED LEARN', SOLTE O BOTO. PISCAR 5 VEZES PARA INDICAR
;    O APAGAMENTO DE TODOS OS CONTROLES GRAVADOS. PARA FUNCIONAR, DEVER SER GRAVADOS 
;    OS CONTROLES DESEJADOS NOVAMENTE.
;    ESTE ARQUIVO  PARTE INTEGRANTE DO BLOG.LARIOS.TECNOLOGIA.WS
;===================================================================================================================
  
	LIST P=12f675 , R=DEC
 	INCLUDE "P12F675.INC"

;PALAVRA DE CONFIGURAO
	ERRORLEVEL      -302   
	__CONFIG		_CP_OFF & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_NOCLKOUT & _BODEN_ON & _MCLRE_OFF

;DEFINIES
#DEFINE		BANK0	BCF	STATUS,RP0 	 ;SETA BANK0 DE MEMORIA
#DEFINE		BANK1	BSF	STATUS,RP0	 ;SETA BANK1

#DEFINE		FLAG	    FLAGS,0          ;FLAG DE SITUAO
#DEFINE     FLAG_ROK    FLAGS,1        ;FLAG DE RECEPO DE TX
#DEFINE		FLAGGR	    FLAGS,2          ;FLAG DE LEARN
#DEFINE		FLAG_PR	    FLAGS,3          ;FLAG DE PRIMEIRA RECEPO
#DEFINE		FLAG_GEEP	FLAGS,4          ;FLAG DE AUTORIZAO DE GRAV. EEPROM
#DEFINE     FLAG_REP    FLAGS,5          ;FLAG DE REPETIO 
;DEFINIES DE PORTAS DE SAIDA
#DEFINE SIN		    GPIO,3  			;PINO 4- IN RF
#DEFINE LRN      	GPIO,5				;PINO 2- IN BOTO LEARN
#DEFINE S3      	GPIO,2				;PINO 5- OUT ON/OFF
#DEFINE LED         GPIO,4   			;PINO 3- OUT LED LEARN
#DEFINE OUT         GPIO,0   			;PINO 7- OUT PWM 0-5V
#DEFINE SW_MODO     GPIO,1              ;PINO 6- IN - MODO DE SAIDA DO PINO 5 (PULSO OU RETENO)

;RAM
     CBLOCK  0X20
 	TEMP,TEMP1,FLAGS,TMP,AUX,HC,LC,BITCOUNTER,STATUSTEMP
	PONT1,PWM,PWM1,TMZ1,TMZ2,TMX1,TMX2,WTEMP,REEP,REEP1	
    RES2,RES1,RES0,RES2A,RES1A,RES0A,RES2B,RES1B,RES0B
    CNT1,CNT0		
     ENDC

;EQUATES RX
TMAX    EQU  .20        ;TEMPO MAXIMO DO SINAL LOW (PERIODO ENTRE TRANSMISSES)
QB      EQU  .24        ;QUANTIDADE DE BYTES A SER RECEBIDO 
MAX_EEPROM EQU  .125    ;ENDEREO PARA PROCURA DE VAGAS PARA GRAVAO DE NOVOS CONTROLES
TPG     EQU  .250       ;TEMPO PARA REFFRESH DO VALOR DA EEPROM APS ATUAO DOS BOTES

;EQUATES PWM

;PASSO_INCREMENTO_PWM  EQU .1 ; 256 PASSOS
PASSO_INCREMENTO_PWM  EQU .4  ; 64 PASSOS

TEMPO_LIGADO_PULSO    EQU .10000  ;ENTRE 8000 A 20000 (evita ficar piscando a saida em modo pulso)

;=========================================================================================================
;                               RESET
;=========================================================================================================
    ORG 0X00
    GOTO START
;=========================================================================================================
;                               INTERRUPO
;=========================================================================================================

    ORG 0X04
    GOTO  INTERRUPO
;=========================================================================================================
;                               INICIO DO PROGRAMA
;=========================================================================================================

START:

	BANK1
    CALL    0X3FF
    MOVWF   OSCCAL          ;CALIBRA OSCILADOR (CERTIFIQUE DE TER BYTE DE CALIBRAO SALVO)
	MOVLW	B'101010'		; SETUP PORTA
	MOVWF   TRISIO
    MOVLW   B'00001000'
    MOVWF   OPTION_REG   ;HABILITA PULL-UPS  TMR0/1
   	CLRF    ANSEL 
   	CLRF	VRCON       ;DESLIGA TENSO DE REFERENCIA
    BSF     WPU,5
    BSF     WPU,1       ;PINO 6 (ESCOLHA DO MODO PULSO OU RETENO PARA TERCEIRO BOTO)
	BANK0
	MOVLW   0X07
	MOVWF	CMCON			;DESLIGA COMPARADORES
	CLRF	GPIO			; RESET PORT
    CLRF    FLAGS   ;APAGA REGISTRADOR DE FLAGS
    MOVLW   .126
    CALL    EEREAD  ;LE EEPROM NO ULTIMO ENDEREO
    ADDLW   .0
    BCF     S3
    BTFSC   STATUS,Z
    BSF     S3
    MOVLW   .127
    CALL    EEREAD  ;LE EEPROM NO ULTIMO ENDEREO
    MOVWF   PWM
    MOVLW   .0
    MOVWF   PWM1
    MOVLW   TPG
    MOVWF   REEP   ;TEMPO DE GRAVAO DO REFRESH DA EEPROM APS ATUAO DE BOTES INC/DEC
    BSF     INTCON,T0IF
    BSF     INTCON,T0IE
    BSF     INTCON,GIE  ;LIGA GERAL DE INTERRUPES

    MOVLW  QB  ;PREPARA PARA A RECEPO COM NUMERO DE BITS A RECEBER
	MOVWF  BITCOUNTER
    BCF   FLAG   ;LIMPA FLAG E REGISTRADORES DE RECEPO
    CLRF  HC
    CLRF  LC
;======================================================================================================
;                           PROGRAMA PRINCIPAL
;======================================================================================================
INICIO_RX:

    DECFSZ CNT0,F
    GOTO   FK4
    DECFSZ   CNT1,F
    GOTO   FK4
    BTFSC  SW_MODO  ; SE EM MODO PULSO
    BCF    S3   ;DESLIGA SAIDA EM MODO PULSO POR TEMPO
FK4

    BTFSC   TMX2,3		;ATRASO PARA BASCULAMENTO MAIS LENTO
    BCF     FLAG_REP    ;LIBERA PARA NOVO ACIONAMENTO DO BOTO ON/OFF

    BTFSS  FLAG_GEEP
    GOTO   SHI23
    DECFSZ  REEP1,F
    GOTO   SHI23
    DECFSZ  REEP,F
    GOTO   SHI23
    BCF     INTCON,GIE  ;DESLIGA GERAL DE INTERRUPES
    MOVFW  PWM          ;SALVA MODIFICAO NA EEPROM
    MOVWF  TMP
    MOVLW  .127
    CALL   EEWRITE  ;GRAVA
    BSF     INTCON,GIE  ;LIGA GERAL DE INTERRUPES
    BCF     FLAG_GEEP

SHI23
    BTFSC   LRN		;TESTA SE BOTO 'LEARN' EST APERTADO
    GOTO   SEG85    ;NO EST, ENTO SEGUE A ROTINA
    BSF    LED      ;EST , ENTO ACENDE 'LED LEARN'
    BSF    FLAGGR   ;SETA FLAG
    CLRF   TMX1     ;LIMPA TEMPORIZADORES
    CLRF   TMX2
    CLRF   TMZ1
    CLRF   TMZ2
    
;TEMPO PARA APAGAMENTO DA EEPROM (TODOS OS CONTROLES GRAVADOS)
LOOP_AP1
    BTFSC   LRN
    GOTO   SEG85
    BTFSS  TMZ2,7		;TESTA SE J DEU O TEMPO DE APAGAMENTO 
    GOTO    LOOP_AP1
    CALL   APAGAMENTO   ;LIMPAR MEMRIA EEPROM E APAGAR CONTROLES
    MOVLW  .5			;NMERO DE PISCADAS PARA INDICAR APAGAMENTO
    MOVWF  TEMP
LOOP_PISCA
    BCF    LED
    CLRF   TMZ1
    CLRF   TMZ2
    BTFSS  TMZ2,3
    GOTO   $-1
    BSF    LED
    CLRF   TMZ1
    CLRF   TMZ2
    BTFSS  TMZ2,3
    GOTO   $-1
    DECFSZ TEMP,F
    GOTO   LOOP_PISCA
    BCF    LED			;APAGA 'LED LEARN'
    BCF    FLAGGR       
    GOTO   INICIO_RX
SEG85
    BTFSS  FLAGGR
    GOTO   SEG99
    BTFSS  TMX2,7   ;TEMPO PARA APAGAMENTO DO LED LEARN APS UM APERTO NO BOTO 'LEARN' 
    GOTO   SEG99
    BCF    FLAGGR   ;APAGA POR TEMPO DE 5SEG.
    BCF    LED
    
  
SEG99
    BTFSS FLAG_ROK		;HOUVE RECEPO DE SINAL?
    GOTO   INICIO_RX 	;NO, VAI PARA O INICIO

RX_IN
    BCF   FLAG_ROK		;HOUVE RECEPO, ENTO TESTA SE FOI A PRIMEIRA
    BTFSC  FLAG_PR      ;FOI A PRIMEIRA RECEPO?
    GOTO   COMPARA      ;SEGUNDA, VAI COMPARAR COM A PRIMEIRA (INTEGRIDADE DO SINAL)
    BSF    FLAG_PR      ;PRIMEIRA
    MOVFW  RES2			;SALVA A PRIMEIRA RECEPO
    MOVWF  RES2A
    MOVFW  RES1
    MOVWF  RES1A
    MOVFW  RES0
    MOVWF  RES0A
    GOTO   INICIO_RX

COMPARA
    MOVFW  RES2			;COMPARA A PRIMEIRA COM A SEGUNDA RECEPO
    XORWF  RES2A,W
    BTFSS  STATUS,Z
    GOTO   ERRO_REC
    MOVFW  RES1
    XORWF  RES1A,W
    BTFSS  STATUS,Z
    GOTO   ERRO_REC
    MOVFW  RES0
    XORWF  RES0A,W
    BTFSS  STATUS,Z
    GOTO   ERRO_REC
    BCF    FLAG_PR
    GOTO   RECEBEU_TX   ;RECEPO CONFIAVEL
ERRO_REC
    CLRF   RES2A        ;FALHA NA RECEPO
    CLRF   RES1A
    CLRF   RES0A
    BCF    FLAG_PR
    GOTO   INICIO_RX
;======================================================================================================
;                   CONFERE SE RECEPO  DE ALGUM CONTROLE PREVIAMENTE GRAVADO NA EEPROM
;======================================================================================================	
RECEBEU_TX   
    
    MOVLW  0XFF
    MOVWF  PONT1
PROX_LOOP
    INCF    PONT1,F			;PROCURA POR CONTROLE RECEBIDO, SE J ESTA GRAVADO NA MEMRIA
    MOVFW   PONT1
    SUBLW    MAX_EEPROM
    BTFSS   STATUS,C
    GOTO    NO_ACHOU       ;NO FOI ENCONTRADO
    MOVFW   PONT1
    CALL    EEREAD
    MOVWF   RES2A
    INCF    PONT1,F
    MOVFW   PONT1
    CALL    EEREAD
    MOVWF   RES1A
    INCF    PONT1,F
    MOVFW   PONT1
    CALL    EEREAD
    XORWF   RES0,W
    BTFSS   STATUS,Z
    GOTO    PROX_LOOP
    MOVFW   RES1A
    XORWF   RES1,W
    BTFSS   STATUS,Z
    GOTO    PROX_LOOP   
    MOVFW   RES2A
    XORWF   RES2,W
    BTFSS   STATUS,Z
    GOTO    PROX_LOOP 
ACHOU
    GOTO    AO
NO_ACHOU
    BTFSS   FLAGGR              ;VERIFICA SE DEVE SER GRAVADO NA EEPROM
    GOTO INICIO_RX              ;NO DEVE
GRAVAR
    MOVLW  0XFF                 ;GRAVAR NA EEPROM
    MOVWF  PONT1				;PROCURA ENDEREO VAZIO PARA GUARDAR CONTROLE
PROX_LOOP1
    INCF    PONT1,F
    MOVFW   PONT1
    SUBLW    MAX_EEPROM
    BTFSS   STATUS,C
    GOTO    INICIO_RX
    MOVFW   PONT1
    CALL    EEREAD
    MOVWF   RES2A
    INCF    PONT1,F
    MOVFW   PONT1
    CALL    EEREAD
    MOVWF   RES1A
    INCF    PONT1,F
    MOVFW   PONT1
    CALL    EEREAD
    XORLW   0XFF
    BTFSS   STATUS,Z
    GOTO    PROX_LOOP1
    MOVFW   RES1A
    XORLW   0XFF
    BTFSS   STATUS,Z
    GOTO    PROX_LOOP1   
    MOVFW   RES2A
    XORLW   0XFF
    BTFSS   STATUS,Z
    GOTO    PROX_LOOP1 

    MOVFW   RES0			;ENDEREO VAZIO ENCONTRADO, GRAVANDO...
    MOVWF   TMP
    MOVFW   PONT1
    CALL    EEWRITE
    MOVFW   RES1
    MOVWF   TMP
    DECF    PONT1,F
    MOVFW   PONT1
    CALL    EEWRITE
    MOVFW   RES2
    MOVWF   TMP
    DECF    PONT1,F
    MOVFW   PONT1
    CALL    EEWRITE
    BCF     FLAGGR			;FINALIZA GRAVAO 
    BCF     LED             ; APAGA 'LED LEARN'
    GOTO    INICIO_RX
;================================================================================================
;                  INCREMENTA/DECREMENTA PWM PARA SAIDA EM MODO 0-5V 
;================================================================================================
AO

    BTFSS   RES2,7      ;TESTA SE AS DUAS CHAVES FORAM ACIONADAS CONCOMITANTEMENTE (OU BOTO 3)
    GOTO    VT9
    BTFSS   RES2,6
    GOTO    VT9

    BTFSC   SW_MODO
    GOTO    PULSO_MODO
;------------------------INICIO : MODO RETENO-------------------------------------------------
RET_MODO
    BTFSC   FLAG_REP   ;TESTA SE BOTO FOI SOLTO PARA HAVER NOVO BASCULAMENTO
    GOTO    DLY
    BSF     FLAG_REP
    BTFSS   S3			;ACIONA SAIDA ON/OFF
    GOTO    LIGAR
    BCF     S3
    MOVLW   0XFF
    MOVWF   TMP
    MOVLW   0X7E
    CALL    EEWRITE
    GOTO    DLY
LIGAR
    BSF     S3
    MOVLW   .0
    MOVWF   TMP
    MOVLW   0X7E
    CALL    EEWRITE
    GOTO    DLY
;----------------------------FIM: MODO RETENO --------------------------------------


;---------------------------INICIO: MODO PULSO --------------------------------------
PULSO_MODO

    BSF     S3
    MOVLW   HIGH (TEMPO_LIGADO_PULSO)
    MOVWF   CNT1
    MOVLW   LOW (TEMPO_LIGADO_PULSO)
    MOVWF   CNT0



;----------------------------FIM : MODO PULSO----------------------------------------
DLY
    

    CLRF    TMX1
    CLRF    TMX2
    GOTO    INICIO_RX

VT9
  
    BTFSS   RES2,6
    GOTO    INCREMENTA_PWM   ;AUMENTAR VCC NA SADA PWM

    BTFSS   RES2,7
    GOTO    DECREMENTA_PWM   ;REDUZIR VCC NA SADA PWM
    GOTO INICIO_RX



INCREMENTA_PWM
    MOVLW   PASSO_INCREMENTO_PWM  
    ADDWF   PWM,W
    BTFSC   STATUS,C
    MOVLW   0XFF              ;VALOR MAXIMO
    MOVWF   PWM
	GOTO 	G_EEP
DECREMENTA_PWM
    MOVLW   PASSO_INCREMENTO_PWM
    SUBWF   PWM,W
    BTFSS   STATUS,C
    MOVLW   0X00			  ;VALOR MNIMO
    MOVWF   PWM
G_EEP
    MOVLW   TPG
    MOVWF   REEP
    BSF     FLAG_GEEP
	GOTO 	INICIO_RX




;================================================================================================
;                  ROTINAS  DE ESCRITA NA  EEPROM
;================================================================================================


EEWRITE
;endereo esta em 'W'
;DADOS A ESCREVER ESTA EM TMP(MSB)  




	
	BANK1
    btfsc EECON1, WR ; ESCRITA ANTERIOR OK?  (ALTERADO)
	goto $-1	 ; (ALTERADO)

	movwf EEADR	
	BANK0

    BCF    INTCON,GIE
    BTFSC  INTCON,GIE
    GOTO   $-2


	MOVFW  TMP		;PEGA PRIMEIRO DADO
	BANK1
	movwf EEDATA
	bcf EECON1, EEIF	
	bsf EECON1, WREN ; HABILITA ESCRITA
	movlw 0x55	
	movwf EECON2	
	movlw 0xAA	
	movwf EECON2	
	bsf EECON1, WR   ;COMEA ESCRITA

    BANK0
    BSF    INTCON,GIE
    RETURN


;================================================================================================
;                  ROTINAS   DE LEITURA DA  EEPROM
;================================================================================================

EEREAD
;endereo esta em 'W'
;DADOS LIDOS SERO ESCRITOS EM TMP0
        

	BANK1
	movwf EEADR	
	bsf EECON1, RD 
    btfsc EECON1, RD 
	goto $-1	
	movf EEDATA,W
	BANK0
	MOVWF	TMP	;RECUPERA PRIMEIRO NUMERO DA EEPROM
	RETURN
;================================================================================================
;                  ROTINA  DE  APAGAMENTO DE CONTROLES REMOTOS GRAVADOS NA  EEPROM
;================================================================================================


APAGAMENTO 

    MOVLW   .0
    MOVWF   TMP
    MOVLW   0X7F
	CALL  	EEWRITE  ;VALOR MINIMO NO PWM

    MOVLW   0XFF
    MOVWF   TMP
	MOVLW 	0X7F		;TOTAL DE 126
	MOVWF	AUX
RET6
	MOVFW	AUX
	MOVLW   .1  ;ACERTA ENDEREO TIRANDO 1
	SUBWF	AUX,W
	CALL  	EEWRITE  ;estava EEWRITE1
	DECFSZ	AUX,F
	GOTO  	RET6
    BCF    S3       ;DESLIGA SAIDA ON/OFF
    CLRF   PWM      ;DESLIGA SAIDA PWM
    RETURN


;;======================================================================================================
;                         ROTINA DE  INTERRUPO DO TIMER 0
;======================================================================================================

INTERRUPO

	MOVWF    WTEMP       		;SALVA 'W'
    SWAPF    STATUS,W    		;SALVA 'STATUS'
    MOVWF    STATUSTEMP
    BANK0
    MOVLW    .140
    MOVWF    TMR0
    INCFSZ   TMX1,F
    GOTO     X4
    INCF     TMX2,F
X4  
    INCFSZ   TMZ1,F
    GOTO     Z4
    INCF     TMZ2,F
Z4  
   
    MOVLW    PASSO_INCREMENTO_PWM
    ADDWF    PWM1,F
    
;;======================================================================================================
;                GERA  PWM C/ TENSO ENTRE 0 E 5VOLTS, NO  PINO 7
;======================================================================================================

SPWM_0_5
    MOVFW   PWM1
    BTFSC   STATUS,Z
    GOTO    SAI_ZERO
    SUBWF   PWM,W
    BTFSC   STATUS,C
    BSF     OUT               ;CANAL PWM NO PINO 7 (0-5 VOLTS)
    BTFSS   STATUS,C
SAI_ZERO
    BCF     OUT
    BCF     INTCON,T0IF         ;APAGA FLAG DO TMR0
 
;;======================================================================================================
;                          SAIDA DA  INTERRUPO 
;======================================================================================================

SAI_INT


    BANK1
    btfsc EECON1, WR ; ESCRITA ANTERIOR OK?  (ALTERADO)
    goto N34
    bcf EECON1, WREN ; DESABILITA ESCRITA
N34
    BANK0


    CALL    RECEIVE             ;EXECUTA ROTINA DE RECEPO DE SINAIS RF
    SWAPF  STATUSTEMP,W   		;RECUPERA 'STATUS'
    MOVWF  STATUS
    SWAPF  WTEMP,F
    SWAPF  WTEMP,W        		;RECUPERA 'W'
	RETFIE
;======================================================================================================
;                ROTINA DE RECEPO RF
;======================================================================================================


RECEIVE
     BTFSC  FLAG_ROK
     RETURN
     BTFSS FLAG        ;TESTA FLAG DE HUM/ZERO RECEBIDO
	 GOTO  L2      ;FLAG=0
     BTFSS SIN      ;SIN ? TESTA SINAL DE ENTRADA DO RECEPTOR DE RF
	 GOTO  M0       ; SIN=0
L2
	 BTFSS SIN      ;SIN ?
     GOTO  LA1      ;SIN=0
	 BSF   FLAG     ;SIN=1
     INCF  HC,F
     BTFSC STATUS,Z
     DECF  HC,F
     RETURN
LA1
    INCF  LC,F     ;INCREMENTA REGISTRADOR DE LOW SIGNAL NA ENTRADA RF
    BTFSC STATUS,Z
    DECF  LC,F
    RETURN
M0
    MOVFW  LC		    ;TESTA SE NO PASSOU DO LIMITE MAXIMO DE TEMPO EM LOW NA ENTRADA DE RF
    SUBLW  TMAX
    BTFSS  STATUS,C
    GOTO   REJ       ;C=0 (-)  -  LC>TMAX (REJEITAR DADO - PAUSA INICIAL)
    MOVFW  HC			;OBTEM O VALOR DE 'CARRY'(HUM OU ZERO)
    SUBWF  LC,W
    RRF RES2B,F
    RRF RES1B,F
    RRF RES0B,F
    DECFSZ  BITCOUNTER,F  ;DECREMENTA O CONTADOR DE BITS A SER RECEBIDO
    GOTO   PROX_LO             ;LE PROXIMO BIT
    GOTO   SEG1
REJ
	MOVLW  QB  ;PREPARA PARA A RECEPO COM NUMERO DE BITS A RECEBER
    MOVWF  BITCOUNTER
PROX_LO
    BCF   FLAG   ;LIMPA FLAG E REGISTRADORES DE RECEPO
    CLRF  HC
    CLRF  LC
    RETURN
SEG1
    MOVFW  RES2B	;SALVA RECEPO
    MOVWF  RES2
    MOVFW  RES1B
    MOVWF  RES1
    MOVFW  RES0B
    MOVWF  RES0
	MOVLW  QB  ;PREPARA PARA A RECEPO COM NUMERO DE BITS A RECEBER
    MOVWF  BITCOUNTER
    BCF   FLAG   ;LIMPA FLAG E REGISTRADORES DE RECEPO
    CLRF  HC
    CLRF  LC
    BSF    FLAG_ROK
    RETURN

;                           FIM DO PROGRAMA
;======================================================================================================
 ORG 0X2100+127
   DW .0
    END
